<?php
/**
 * Josetta - native multilingual manager for Joomla!
 *
 * @author      Yannick Gaultier
 * @copyright   (c) Yannick Gaultier - Weeblr llc - 2015
 * @package     josetta
 * @license     http://www.gnu.org/copyleft/gpl.html GNU/GPL
 * @version     2.3.1.643
 * @date		2015-04-05
 */

defined('JPATH_BASE') or die;

/**
 * Supports a modal article picker.
 *
 * @package		Joomla.Administrator
 * @subpackage	com_content
 * @since		1.6
 */
class JFormFieldModal_Article extends JFormField
{
  /**
   * The form field type.
   *
   * @var		string
   * @since	1.6
   */
  protected $type = 'Modal_Article';

  /**
   * Method to get the field input markup.
   *
   * @return	string	The field input markup.
   * @since	1.6
   */
  protected function getInput()
  {
    // Load the modal behavior script.
    JHtml::_('behavior.modal', 'a.modal');

    // Build the script.
    $script = array();
    $script[] = '	function jSelectArticle_'.$this->id.'(id, title, catid, object) {';
    $script[] = '		document.id("'.$this->id.'_id").value = id;';
    $script[] = '		document.id("'.$this->id.'_name").value = title;';
    $script[] = '		SqueezeBox.close();';
    $script[] = '	}';

    // Add the script to the document head.
    JFactory::getDocument()->addScriptDeclaration(implode("\n", $script));


    // Setup variables for display.
    $html	= array();
    $link	= 'index.php?option=com_josetta&amp;view=articles&amp;layout=modal&amp;Itemid='.JosettaHelper::getJosettaItemid().'&amp;tmpl=component&amp;function=jSelectArticle_'.$this->id;

    $db	= ShlDbHelper::getDb();
    $db->setQuery(
        'SELECT title' .
        ' FROM #__content' .
        ' WHERE id = '.(int) $this->value
    );
    
    try {
      $title = $db->shlLoadResult();
    } catch (Exception $e) {
      ShlSystem_Log::error( 'josetta', '%s::%s::%d: %s', __CLASS__, __METHOD__, __LINE__, $e->getMessage());
      JError::raiseWarning(500, $e->getMessage());
    }

    if (empty($title)) {
      $title = JText::_('COM_CONTENT_SELECT_AN_ARTICLE');
    }
    $title = htmlspecialchars($title, ENT_QUOTES, 'UTF-8');

    // The current user display field.
    $html[] = '<div class="fltlft">';
    $html[] = '  <input type="text" id="'.$this->id.'_name" value="'.$title.'" disabled="disabled" size="35" />';
    $html[] = '</div>';

    // The user select button.
    $html[] = '<div class="button2-left">';
    $html[] = '  <div class="blank">';
    $html[] = '	<a class="modal" title="'.JText::_('COM_CONTENT_CHANGE_ARTICLE').'"  href="'.$link.'&amp;'.JSession::getFormToken().'=1" rel="{handler: \'iframe\', size: {x: 900, y: 450}}">'.JText::_('COM_CONTENT_CHANGE_ARTICLE_BUTTON').'</a>';
    $html[] = '  </div>';
    $html[] = '</div>';

    // The active article id field.
    if (0 == (int)$this->value) {
      $value = '';
    } else {
      $value = (int)$this->value;
    }

    // class='required' for client side validation
    $class = '';
    if ($this->required) {
      $class = ' class="required modal-value"';
    }

    $html[] = '<input type="hidden" id="'.$this->id.'_id"'.$class.' name="'.$this->name.'" value="'.$value.'" />';

    return implode("\n", $html);
  }
}
